component {

	// Messaging
	property name="messagebox" 	inject="messageBox@cbMessagebox";
	// File Utils
	property name="fileUtils"	inject="FileUtils@cbcommons";
	// S3 SDK
	property name="s3" 			inject="AmazonS3@s3sdkexplorer";

	// Handler Properties
	this.preHandler_except = "authenticate";

	/**
	* Executes before all handler actions
	*/
	any function preHandler( event, rc, prc, action, eventArguments ){
		// module root
		prc.root = event.getModuleRoot();
	}

	/**
	* authenticate
	*/
	function authenticate( event, rc, prc ){
		if( event.valueExists('authButton') ){
			// Save credentials
			setSetting("s3_accessKey",rc.accessKey);
			setSetting("s3_secretKey",rc.secretKey);
			variables.s3.setAuth(rc.accessKey,rc.secretKey);
			// relocate
			relocate('s3sdkexplorer/explorer');
		}
		
		event.setView("explorer/authenticate");
	}

	/**
	* index
	*/
	function index( event, rc, prc ){
		prc.allBuckets = variables.s3.listBuckets();
		event.setView( "explorer/index" );
	}

	/**
	* createBucket
	*/
	function createBucket( event, rc, prc ){
		variables.s3.putBucket( rc.bucketName, rc.acl, rc.storage );
		variables.messagebox.info( message="bucket created" );
		relocate( "s3sdkexplorer" );
	}
	
	/**
	* removeBucket
	*/
	function removeBucket( event, rc, prc ){


		if( variables.s3.deleteBucket( rc.bucketName ) ){
			variables.messagebox.info( message="Bucket Removed!" );
		} else {
			variables.messagebox.error( message="S3 service could not remove bucket. Please refer to debugging information." );
		}
		relocate( "s3sdkexplorer" );
	}	
	
	/**
	* viewBucket
	*/
	function viewBucket( event, rc, prc ){
		event.paramValue( "delimiter", "/" )
			.paramValue( "folderName", "" );
		// do we have a folder to display?
		if( len( rc.folderName ) > 0 ){
			rc.foldername = replacenocase( rc.foldername, "|", "/", "all" ) & '/';
		}	
		prc.allObjects = variables.s3.getBucket(
			bucketName 	= replace( rc.bucketName, "|", "/", "all" ), 
			delimiter 	= "#rc.delimiter#", 
			prefix 		= "#rc.folderName#"
		);  		
		prc.allBuckets = variables.s3.listBuckets();
	}

	/**
	* genAuthenticatedURL
	*/
	function genAuthenticatedURL( event, rc, prc ){
		rc.timedLink = variables.s3.getAuthenticatedURL( bucketName=rc.bucketname, uri=rc.key, virtualHostStyle=true );
		event.renderData( data=renderView("explorer/genAuthenticatedURL") );
	}
	
	/**
	* getObjectInfo
	*/
	function getObjectInfo( event, rc, prc ){
		if(structKeyExists(rc, 'isDirectory') AND rc.isDirectory EQ true){
			rc.objectKey = rc.objectKey& "_$folder$";
		}
		rc.info = variables.s3.getObjectInfo( rc.bucketname, rc.objectKey);
		event.renderData( data=renderView( "explorer/objectInfo" ) );
	}
	
	/**
	* upload
	*/
	function upload( event, rc, prc ){

		var tempDir 	= expandPath(prc.root&'/_tmp/');

		if(NOT directoryExists(tempDir)){
			directoryCreate(tempDir);
		}

		var fileInfo 	= fileUtils.uploadFile(
			fileField 		= "fileobject",
			destination 	= tempDir,
		 	mode 			= "666",
		 	NameConflict 	= "overwrite"
		 );
		var returnFolderPath = "";
		if ( listLen( rc.foldername ) ){
			returnFolderPath = replacenocase( reverse( replacenocase( reverse( rc.foldername ), "/", "", "one" ) ), "/", "|", "all" );
		}
		// Inflate json metadata
		if( NOT len( event.getTrimValue( "extraMetadata" ) ) ){ 
			rc.extrametadata = "{}"; 
		}
		rc.extrametadata = deserializeJSON( rc.extrametadata );
		
		try{
			rc.results = variables.s3.putObjectFile(
				bucketName 		= rc.bucketName,
				filePath 		= fileInfo.serverDirectory & "/" & fileInfo.serverFile,
				uri 			= rc.foldername & getFileFromPath( fileInfo.serverDirectory & "/" & fileInfo.serverFile ),
				contentType 	= fileInfo.contentType,
				cacheControl 	= event.getTrimValue( "cc" ),
				expires 		= event.getTrimValue( "expires" ),
				acl 			= rc.acl,
				metaHeaders 	= rc.extrametadata
			);
			variables.messagebox.info( message="Uploaded file with etag: #rc.results#");
		} catch( Any e ) {
			variables.messagebox.error( message="Error uploading file: #e.message# #e.detail#");
			log.error("Error putting file to S3",e);
		} finally {
			// remove file
			fileDelete( "#tempDir#/#fileInfo.serverFile#" );
		}
		
		// relocate
		relocate( "s3sdkexplorer/bucket/#urlEncodedFormat( rc.bucketName )#/#urlEncodedFormat( returnFolderPath )#" );
	}
	
	/**
	* createFolder
	*/
	function createFolder( event, rc, prc ){

		try{
			rc.results = variables.s3.putObjectFolder(
				bucketName 	= rc.bucketName,
				uri  		= rc.foldername & "_$folder$" ,
				acl 		= rc.acl
			);
										  
			variables.messagebox.info( message="Uploaded Folder with etag: #rc.results#");
		} catch(Any e) {
			variables.messagebox.error( message="Error uploading Folder: #e.message# #e.detail#" );
			log.error( "Error putting file to S3", e );
		}    
		
		// relocate
		relocate( "s3sdkexplorer/bucket/#urlEncodedFormat( rc.bucketName )#/#urlEncodedFormat( replacenocase( rc.foldername, '/', '|', 'all' ) )#" );
	}
    
 	/**
 	* removeObject
 	*/
 	function removeObject( event, rc, prc ){
		var returnFolderPath = "";
		if ( listLen( rc.uri ) ){
			returnFolderPath = listrest( reverse( urlDecode( rc.uri ) ), "/" );
			returnFolderPath = reverse( replacenocase( returnFolderPath, "/", "|", "all") );
		}

		if( variables.s3.deleteObject( rc.bucketName, rc.uri ) ){
			variables.messagebox.info( "#encodeForHTML( rc.uri )# Removed!" );
		} else {
			variables.messagebox.error( "S3 service could not remove object. Please refer to debugging information." );
		}
		relocate( event="s3sdkexplorer/bucket/#urlEncodedFormat( rc.bucketName )#/#urlEncodedFormat( returnFolderPath )#" );
 	}

 	/**
 	* copyDialog
 	*/
 	function copyDialog( event, rc, prc ){
 		rc.allBuckets = variables.s3.listBuckets();
		event.renderData( data = renderView( "explorer/copyDialog" ) );
 	}
	
	/**
	* copyObject
	*/
	function copyObject( event, rc, prc ){
		// Inflate json metadata
		if( len( event.getTrimValue( "extraMetadata" ) ) ){ 
			rc.extrametadata = "{}"; 
		}
		rc.extrametadata = deserializeJSON( rc.extrametadata );
		
		// Copy
		rc.results = variables.s3.copyObject(
			fromBucket 		= rc.fromBucket,
			fromURI 		= rc.fromURI,
			toBucket 		= rc.toBucket,
			toURI 			= rc.toURI,
			acl 			= rc.acl,
			copymetaData 	= rc.copyMetadata,
			metaHeaders 	= rc.extrametadata
		);
		
		variables.messagebox.info( "Object copied successfully" );
		
		// relocate
		relocate( event="s3sdkexplorer/bucket/#urlEncodedFormat( rc.fromBucket )#" );
	}
	
	/**
	* objectACL
	*/
	function objectACL( event, rc, prc ){
		if(structKeyExists(rc, 'isDirectory') AND rc.isDirectory EQ true){
			//rc.objectName = rc.objectName& "_$folder$/";
		}
		rc.grants = variables.s3.getAccessControlPolicy( rc.objectName );    	
		event.renderData( data=renderView( 'explorer/objectACL' ) );
	}

}